import {
  select,
  scaleBand,
  scaleLinear,
  axisLeft,
  axisBottom,
} from 'd3';

export const viz = (
  svg,
  {
    data,
    xValue,
    xAxisLabelText,
    xAxisLabelOffset,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginTop,
    marginRight,
    marginBottom,
    width,
    height,
    innerRectFill,
  },
) => {
  const innerWidth = width - marginLeft - marginRight;
  const innerHeight = height - marginTop - marginBottom;

  const chestPainTypeDescriptions = {
    TA: 'Typical Angina',
    ATA: 'Atypical Angina',
    NAP: 'Non-Anginal Pain',
    ASY: 'Asymptomatic',
  };

  const x0Scale = scaleBand()
    .domain(
      data.map(
        (d) =>
          chestPainTypeDescriptions[d.ChestPainType] ||
          d.ChestPainType,
      ),
    )
    .rangeRound([0, innerWidth])
    .paddingInner(0.1);

  const x1Scale = scaleBand()
    .domain(['M', 'F'])
    .rangeRound([0, x0Scale.bandwidth()])
    .padding(0.05);

  const yScale = scaleLinear()
    .domain([
      0,
      Math.max(...data.flatMap((d) => [d.M, d.F])),
    ])
    .rangeRound([innerHeight, 0]);

  svg
    .append('rect')
    .attr('x', marginLeft)
    .attr('y', marginTop)
    .attr('width', innerWidth)
    .attr('height', innerHeight)
    .attr('fill', innerRectFill);

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${marginLeft},${marginTop})`,
    );

  g.append('g').call(axisLeft(yScale));
  g.append('g')
    .call(axisBottom(x0Scale))
    .attr('transform', `translate(0,${innerHeight})`);

  
  const tooltip = select('body')
    .append('div')
    .attr('class', 'tooltip')
    .style('opacity', 0)
    .style('position', 'absolute')
    .style('text-align', 'center')
    .style('width', 'auto')
    .style('padding', '8px')
    .style('font', '12px sans-serif')
    .style('background', 'lightsteelblue')
    .style('border', '0px')
    .style('border-radius', '8px')
    .style('pointer-events', 'none');

  const chestPainType = g
    .selectAll('.chestPainType')
    .data(data)
    .enter()
    .append('g')
    .attr(
      'transform',
      (d) =>
        `translate(${x0Scale(chestPainTypeDescriptions[d.ChestPainType] || d.ChestPainType)},0)`,
    );

  chestPainType
    .selectAll('.bar')
    .data((d) =>
      ['M', 'F'].map((key) => ({ key, value: d[key] })),
    )
    .enter()
    .append('rect')
    .attr('x', (d) => x1Scale(d.key))
    .attr('y', (d) => yScale(d.value))
    .attr('width', x1Scale.bandwidth())
    .attr('height', (d) => innerHeight - yScale(d.value))
    .attr('fill', (d) =>
      d.key === 'M' ? 'steelblue' : 'pink',
    )
    .on('mouseover', function (event, d) {
      select(this)
        .attr('fill', 'orange')
        .attr('stroke', 'yellow')
        .attr('stroke-width', '2');
      tooltip
        .transition()
        .duration(200)
        .style('opacity', 0.9);
      tooltip
        .html(`Count: ${d.value}`)
        .style('left', event.pageX + 'px')
        .style('top', event.pageY - 28 + 'px');
    })
    .on('mouseout', function (event, d) {
      select(this)
        .attr('fill', d.key === 'M' ? 'steelblue' : 'pink')
        .attr('stroke', null)
        .attr('stroke-width', null);
      tooltip
        .transition()
        .duration(500)
        .style('opacity', 0);
    });

  
  const legend = svg
    .append('g')
    .attr(
      'transform',
      `translate(${width - marginRight - 100},${marginTop})`,
    );

  legend
    .selectAll('rect')
    .data([
      { color: 'steelblue', gender: 'M' },
      { color: 'pink', gender: 'F' },
    ])
    .enter()
    .append('rect')
    .attr('x', 0)
    .attr('y', (d, i) => i * 20)
    .attr('width', 18)
    .attr('height', 18)
    .attr('fill', (d) => d.color);

  legend
    .selectAll('text')
    .data([
      { color: 'steelblue', gender: 'M' },
      { color: 'pink', gender: 'F' },
    ])
    .enter()
    .append('text')
    .attr('x', 24)
    .attr('y', (d, i) => i * 20 + 14)
    .text((d) => d.gender);

  
  svg
    .append('text')
    .attr('class', 'axis-label')
    .attr('transform', 'rotate(-90)')
    .attr('y', marginLeft - 50)
    .attr('x', 0 - height / 2)
    .attr('dy', '1em')
    .style('text-anchor', 'middle')
    .text(yAxisLabelText);

  svg
    .append('text')
    .attr('class', 'axis-label')
    .attr('x', width / 2)
    .attr('y', height - 10)
    .attr('text-anchor', 'middle')
    .text(xAxisLabelText);
};
